@@ -6,6 +6,7 @@ Parser for HTML forms, that fills in defaults and errors.  See
 import HTMLParser
 import cgi
 import re
+from htmlentitydefs import name2codepoint
 
 __all__ = ['render', 'htmlliteral', 'default_formatter',
            'none_formatter', 'escape_formatter',
@@ -14,7 +15,8 @@ __all__ = ['render', 'htmlliteral', 'default_formatter',
 def render(form, defaults=None, errors=None, use_all_keys=False,
            error_formatters=None, add_attributes=None,
            auto_insert_errors=True, auto_error_formatter=None,
-           text_as_default=False, listener=None):
+           text_as_default=False, listener=None, encoding=None,
+           error_class='error'):
     """
     Render the ``form`` (which should be a string) given the defaults
     and errors.  Defaults are the values that go in the input fields
@@ -52,6 +54,9 @@ def render(form, defaults=None, errors=None, use_all_keys=False,
 
     ``listener`` can be an object that watches fields pass; the only
     one currently is in ``htmlfill_schemabuilder.SchemaBuilder``
+    
+    ``encoding`` specifies an encoding to assume when mixing str and 
+    unicode text in the template.
     """
     if defaults is None:
         defaults = {}
@@ -64,7 +69,8 @@ def render(form, defaults=None, errors=None, use_all_keys=False,
         add_attributes=add_attributes,
         auto_error_formatter=auto_error_formatter,
         text_as_default=text_as_default,
-        listener=listener,
+        listener=listener, encoding=encoding,
+        error_class=error_class,
         )
     p.feed(form)
     p.close()
@@ -166,7 +172,7 @@ class FillingParser(HTMLParser.HTMLParser):
                  error_formatters=None, error_class='error',
                  add_attributes=None, listener=None,
                  auto_error_formatter=None,
-                 text_as_default=False):
+                 text_as_default=False, encoding=None):
         HTMLParser.HTMLParser.__init__(self)
         self._content = []
         self.source = None
@@ -193,8 +199,10 @@ class FillingParser(HTMLParser.HTMLParser):
         self.listener = listener
         self.auto_error_formatter = auto_error_formatter
         self.text_as_default = text_as_default
+        self.encoding = encoding
 
     def feed(self, data):
+        self.data_is_str = isinstance(data, str)
         self.source = data
         self.lines = data.split('\n')
         self.source_pos = 1, 0
@@ -203,6 +211,7 @@ class FillingParser(HTMLParser.HTMLParser):
         HTMLParser.HTMLParser.feed(self, data)
 
     def close(self):
+        self.handle_misc(None)
         HTMLParser.HTMLParser.close(self)
         unused_errors = self.errors.copy()
         for key in self.used_errors.keys():
@@ -230,12 +239,57 @@ class FillingParser(HTMLParser.HTMLParser):
                 assert False, (
                     "These errors were not used in the form: %s" % 
                     ', '.join(error_text))
-        self._text = ''.join([
-            t for t in self._content if not isinstance(t, tuple)])
+        if self.encoding is not None:
+            new_content = []
+            for item in self._content:
+                if isinstance(item, str):
+                    item = item.decode(self.encoding)
+                new_content.append(item)
+            self._content = new_content
+        try:
+            self._text = ''.join([
+                t for t in self._content if not isinstance(t, tuple)])
+        except UnicodeDecodeError, e:
+            if self.data_is_str:
+                e.reason += (
+                    " the form was passed in as an encoded string, but "
+                    "some data or error messages were unicode strings; "
+                    "the form should be passed in as a unicode string")
+            else:
+                e.reason += (
+                    " the form was passed in as an unicode string, but "
+                    "some data or error message was an encoded string; "
+                    "the data and error messages should be passed in as "
+                    "unicode strings")
+            raise
 
     def add_key(self, key):
         self.used_keys[key] = 1
 
+    _entityref_re = re.compile('&([a-zA-Z][-.a-zA-Z\d]*);')
+    _charref_re = re.compile('&#(\d+|[xX][a-fA-F\d]+);')
+
+    def unescape(self, s):
+        s = self._entityref_re.sub(self._sub_entityref, s)
+        s = self._charref_re.sub(self._sub_charref, s)
+        return s
+
+    def _sub_entityref(self, match):
+        name = match.group(1)
+        if name not in name2codepoint:
+            # If we don't recognize it, pass it through as though it
+            # wasn't an entity ref at all
+            return match.group(0)
+        return unichr(name2codepoint[name])
+
+    def _sub_charref(self, match):
+        num = match.group(1)
+        if num.lower().startswith('x'):
+            num = int(num[1:], 16)
+        else:
+            num = int(num)
+        return unichr(num)
+
     def handle_starttag(self, tag, attrs, startend=False):
         self.write_pos()
         if tag == 'input':
