@@ -52,9 +52,10 @@ class DecodeCase(object):
         all_cases.append(self)
 
     def test(self):
-        print self.raw_input
+        print 'input', repr(self.input)
         actual = self.schema.to_python(self.input)
-        assert actual == self.output        
+        print 'output', repr(actual)
+        assert actual == self.output
 
 class BadCase(DecodeCase):
 
@@ -66,9 +67,9 @@ class BadCase(DecodeCase):
             self.output = self.output['text']
 
     def test(self):
-        print self.raw_input
+        print repr(self.raw_input)
         try:
-            self.schema.to_python(self.input)
+            print repr(self.schema.to_python(self.input))
         except Invalid, e:
             actual = e.unpack_errors()
             assert actual == self.output
@@ -133,7 +134,7 @@ BadCase(AddressesForm,
 def test_this():
 
     for case in all_cases:
-        yield case.test
+        yield (case.test,)
 
 
 def test_merge():
@@ -148,4 +149,19 @@ def test_merge():
             == dict(a=['a1\naa1', 'a2'], b=['b\nbb', 'bbb'],
                     c=['c']))
 
+class ChainedTest(Schema):
+    a = validators.String()
+    a_confirm = validators.String()
 
+    b = validators.String()
+    b_confirm = validators.String()
+
+    chained_validators = [validators.FieldsMatch('a', 'a_confirm'),
+                            validators.FieldsMatch('b', 'b_confirm')]
+
+def test_multiple_chained_validators_errors():
+    try:
+        s = ChainedTest()
+        s.to_python({'a':'1', 'a_confirm':'2', 'b':'3', 'b_confirm':'4'})
+    except Invalid, e:
+        assert(e.error_dict.has_key('a_confirm') and e.error_dict.has_key('b_confirm'))
